import Ogma, { RawGraph } from "@linkurious/ogma";
import Prism from "prismjs";
import "prismjs/themes/prism.css";
import "./styles.css";
import { isGenre, isMovie, isPerson, NodeDataType } from "./types";
import * as Queries from "./queries";
import { APIClient as API } from "./client";
import { addStyles } from "./styles";

const ogma = new Ogma<NodeDataType>({
  container: "graph-container",
  options: {
    backgroundColor: "transparent",
  },
});
addStyles(ogma);

const client = new API();
const initialMovie = "Chappie";
const graph = await client.getMovieByTitle(initialMovie);
setQuery(Queries.GET_MOVIE_BY_TITLE.replace("@title", `"${initialMovie}"`));

await ogma.setGraph(graph);
await ogma.layouts.force({ locate: true });

async function addGraphAndZoomIn(graph: RawGraph) {
  if (graph.nodes.length === 0) return;
  await ogma.addGraph(graph, { ignoreInvalid: true });
  await ogma.layouts.force({ gpu: true });
  await ogma.view.moveToBounds(
    ogma
      .getNodes(graph.nodes.map((n) => n.id!))
      .getBoundingBox()
      .pad(50),
    { duration: 200 }
  );
}

ogma.events.on("doubleclick", async ({ target }) => {
  if (target && target.isNode) {
    if (isMovie(target)) {
      const movieTitle = target.getData("title");
      setQuery(Queries.GET_MOVIE_BY_TITLE.replace("@title", `"${movieTitle}"`));
      const graph = await client.getMovieByTitle(movieTitle);
      await ogma.addGraph(graph, { ignoreInvalid: true });
      await addGraphAndZoomIn(graph);
    } else if (isPerson(target)) {
      const actorName = target.getData("name");
      setQuery(
        Queries.GET_MOVIES_BY_ACTOR_NAME.replace("@actorName", `"${actorName}"`)
      );
      const graph = await client.getActorByName(actorName);
      await addGraphAndZoomIn(graph);
    } else if (isGenre(target)) {
      const genreName = target.getData("name");
      setQuery(
        Queries.GET_MOVIES_BY_GENRE_NAME.replace("@genreName", `"${genreName}"`)
      );
      const graph = await client.getGenreByName(genreName);
      await addGraphAndZoomIn(graph);
    }
  }
});

function setQuery(query: string) {
  const codeBlock = document.querySelector<HTMLElement>("#query-text")!;
  codeBlock.innerHTML = query;
  Prism.highlightElement(codeBlock);
}

document.getElementById("layout")!.addEventListener("click", async () => {
  await ogma.layouts.force({ gpu: true });
});
