import { Rows } from '@google-cloud/spanner/build/src/transaction';
import dotenv from 'dotenv';
import express, { Request, Response } from 'express';
import { getDatabase } from './db';
import { Parser } from './parser';
import {
  GET_MOVIE_BY_TITLE,
  GET_MOVIES_BY_ACTOR_NAME,
  GET_MOVIES_BY_DIRECTOR_NAME,
  GET_MOVIES_BY_GENRE_NAME
} from './queries';

// Load environment variables from .env file
dotenv.config();

// Initialize Express app
const app = express();
const port = 3000;

const database = getDatabase();
const parser = new Parser();

const toGraph = (rows: Rows) => parser.parse(rows.map(r => r.toJSON()));

// Route stubs
app.get('/movie/:title', async (req: Request, res: Response) => {
  const { title } = req.params;
  const [rows] = await database.run({
    sql: GET_MOVIE_BY_TITLE,
    params: { title }
  });
  // Placeholder for database query logic
  res.json({
    message: `Stub: Get movie with title "${title}"`,
    graph: toGraph(rows)
  });
});

app.get('/actor/:name', async (req: Request, res: Response) => {
  const { name } = req.params;
  const [rows] = await database.run({
    sql: GET_MOVIES_BY_ACTOR_NAME,
    params: { actorName: name }
  });

  // Placeholder for database query logic
  res.json({
    message: `Stub: Get actor with name "${name}"`,
    graph: toGraph(rows)
  });
});

app.get('/genre/:name', async (req: Request, res: Response) => {
  const { name } = req.params;
  const [rows] = await database.run({
    sql: GET_MOVIES_BY_GENRE_NAME,
    params: { genreName: name }
  });

  // Placeholder for database query logic
  res.json({
    message: `Stub: Get genre with name "${name}"`,
    graph: toGraph(rows)
  });
});

app.get('/director/:name', async (req: Request, res: Response) => {
  const { name } = req.params;
  const [rows] = await database.run({
    sql: GET_MOVIES_BY_DIRECTOR_NAME,
    params: { directorName: name }
  });
  // Placeholder for database query logic
  res.json({
    message: `Stub: Get director with name "${name}"`,
    graph: toGraph(rows)
  });
});

// Start the server
app.listen(port, () => {
  console.log(`Server is running on http://localhost:${port}`);
});
